@@ -4,7 +4,6 @@
 //!
 #![deny(unused_extern_crates)]
 #![warn(
-    box_pointers,
     clippy::all,
     clippy::nursery,
     clippy::pedantic,
@@ -25,6 +24,8 @@
 #[allow(non_camel_case_types)]
 #[allow(non_snake_case)]
 #[allow(non_upper_case_globals)]
+#[allow(unused_qualifications)]
+#[allow(clippy::missing_const_for_fn)]
 mod bindings {
     // In `build.rs`, we use `bindgen` to generate bindings based on C headers
     // and `libprism`. Here is where we pull in those bindings and make
